Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var reactSpring = require('react-spring');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

// Keeps track of whether this is the first time render since the component was mounted.
// shamelessly taken from https://github.com/streamich/react-use
function useFirstMount() {
    var isFirst = React.useRef(true);
    if (isFirst.current) {
        isFirst.current = false;
        return true;
    }
    return isFirst.current;
}

var Accordion = function (_a) {
    var _b = _a.className, className = _b === void 0 ? '' : _b, children = _a.children, _c = _a.toggled, toggled = _c === void 0 ? false : _c, onFullyShown = _a.onFullyShown, onFullyHidden = _a.onFullyHidden, props = __rest(_a, ["className", "children", "toggled", "onFullyShown", "onFullyHidden"]);
    var isFirstMount = useFirstMount();
    var handleAnimationRest = React.useCallback(function () {
        if (toggled && onFullyShown)
            onFullyShown();
        else if (!toggled && onFullyHidden)
            onFullyHidden();
    }, [toggled, onFullyShown, onFullyHidden]);
    var cssClass = ["image-tooltips-item__content", className];
    return (React.createElement(reactSpring.Spring, { config: {
            tension: 170,
            friction: 22,
            clamp: true
        }, from: { height: 0 }, to: { height: toggled ? 'auto' : 0 }, immediate: isFirstMount, onRest: handleAnimationRest }, function (styleProps) { return (React.createElement(reactSpring.animated.div, __assign({ className: cssClass.join(' '), style: styleProps }, props), children)); }));
};

var ImageTooltipsTrigger = function (_a) {
    var className = _a.className, children = _a.children, props = __rest(_a, ["className", "children"]);
    var cssClass = ["image-tooltips-item__trigger", className];
    return (React.createElement("div", __assign({ className: cssClass.join(' ') }, props, { tabIndex: 0 }), children));
};

var ImageTooltipsItem = function () {
    return React.createElement("div", null);
};
var _ImageTooltipsItem = function (_a) {
    var children = _a.children, dataId = _a.dataId, top = _a.top, left = _a.left, imageSize = _a.imageSize, toggled = _a.toggled, trigger = _a.trigger, props = __rest(_a, ["children", "dataId", "top", "left", "imageSize", "toggled", "trigger"]);
    var cssClass = ['image-tooltips-item'];
    if (toggled) {
        cssClass.push('toggled');
    }
    // Validation of tooltip coordinates
    var validateCoord = function (c, max) {
        if (c <= 0) {
            console.log("Tooltip with id " + dataId + " has coordinate outside of image.");
            return 1;
        }
        if (c > max) {
            console.log("Tooltip with id " + dataId + " has coordinate outside of image.");
            return max;
        }
        return c;
    };
    left = validateCoord(left, imageSize.initW);
    top = validateCoord(top, imageSize.initH);
    // Calculate tooltip position based on image scaling
    var style = {
        left: Math.round(left * imageSize.curW / imageSize.initW) + "px",
        top: Math.round(top * imageSize.curH / imageSize.initH) + "px"
    };
    return (React.createElement("div", { className: cssClass.join(' '), style: style, "data-id": dataId },
        trigger ? (React.createElement(ImageTooltipsTrigger, __assign({}, trigger.props), trigger.props.children)) : (React.createElement(ImageTooltipsTrigger, null, "+")),
        React.createElement(Accordion, __assign({ toggled: toggled }, props), children)));
};

var ImageTooltips = function (_a) {
    var children = _a.children, width = _a.width, height = _a.height, _b = _a.triggerEvent, triggerEvent = _b === void 0 ? 'click' : _b, props = __rest(_a, ["children", "width", "height", "triggerEvent"]);
    var _c = React.useState({ imageSize: null, toggled: null }), state = _c[0], _setState = _c[1];
    var stateRef = React.useRef(state);
    var imageEl = React.useRef(null);
    var divEl = React.useRef(null);
    // Need to use Ref because state is unavailable in event listeners
    var setImageSize = function (imageSize) {
        stateRef.current.imageSize = imageSize;
        _setState(__assign({}, state, { imageSize: imageSize }));
    };
    var setToggled = function (toggled) {
        stateRef.current.toggled = toggled;
        _setState(__assign({}, state, { toggled: toggled }));
    };
    var timerId;
    React.useEffect(function () {
        // Listener that handles resize with simple throttling
        var handleResize = function () {
            if (timerId) {
                return;
            }
            timerId = global.setTimeout(function () {
                setImageSize({
                    initW: stateRef.current.imageSize.initW,
                    initH: stateRef.current.imageSize.initH,
                    curW: imageEl.current.offsetWidth,
                    curH: imageEl.current.offsetHeight
                });
                timerId = undefined;
            }, 200);
        };
        window.addEventListener('resize', handleResize);
        return function () { return window.removeEventListener('resize', handleResize); };
    }, []);
    // Add event handler for all tooltip items
    React.useEffect(function () {
        var startHandler = function (event) {
            var clicked = event.target;
            if (clicked.classList.contains('image-tooltips-item__trigger')) {
                var dataIdStr = clicked.parentElement.getAttribute('data-id');
                if (dataIdStr) {
                    var dataId = parseInt(dataIdStr);
                    setToggled(stateRef.current.toggled != dataId ? dataId : null);
                }
                else {
                    setToggled(null);
                }
            }
        };
        var endHandler = function () {
            setToggled(null);
        };
        divEl.current.addEventListener(triggerEvent, startHandler);
        if (triggerEvent == 'mouseover') {
            divEl.current.addEventListener('mouseout', endHandler);
        }
        return function () {
            divEl.current.removeEventListener(triggerEvent, startHandler);
            if (triggerEvent == 'mouseover') {
                divEl.current.removeEventListener('mouseout', endHandler);
            }
        };
    }, []);
    // Making image responsive
    var style = {
        maxWidth: width + "px"
    };
    return (React.createElement("div", { ref: divEl, className: "image-tooltips", style: style },
        React.createElement("img", __assign({ ref: imageEl }, props, { onLoad: function () { return setImageSize({
                // Set initial image dimensions in state
                initW: width,
                initH: height,
                curW: imageEl.current.offsetWidth,
                curH: imageEl.current.offsetHeight
            }); } })),
        state.imageSize && children && React.Children.map(children, function (child, index) {
            if (!child || !React.isValidElement(child)) {
                return child;
            }
            var childEl = child;
            return (React.createElement(_ImageTooltipsItem, __assign({ key: index, dataId: index, toggled: index === state.toggled, imageSize: state.imageSize }, childEl.props), childEl.props.children));
        })));
};

exports.ImageTooltips = ImageTooltips;
exports.ImageTooltipsItem = ImageTooltipsItem;
exports.ImageTooltipsTrigger = ImageTooltipsTrigger;
//# sourceMappingURL=index.js.map
